#include "ctextarea.h"
#include <windows.h>

extern void* hInstance;
inline HINSTANCE GetInstance () { return (HINSTANCE)hInstance; }

BEGIN_NAMESPACE_VSTGUI
extern long gStandardFontSize [];
extern const char *gStandardFontName [];
END_NAMESPACE_VSTGUI

CTextArea::CTextArea (const CRect &size, const long style)
:	CControl (size, 0, 0, 0), style (style)
{
	backOffset (0, 0);

	fontID      = kNormalFont;
	txtFace     = kNormalFace;
	fontColor   = kWhiteCColor;
	backColor   = kBlackCColor;
	if (style & kNoDrawStyle)
		setDirty (false);
}

CTextArea::~CTextArea ()
{}

void CTextArea::draw (CDrawContext *pContext)
{
	if (platformControl)
	{
		SetWindowText((HWND)platformControl , text);
		setDirty (false);
		return;
	}

	CRect rect (size);
	CView* parent = getParentView ();
	CRect vSize;
	while (parent)
	{
		if (parent->notify (this, kMsgCheckIfViewContainer) == kMessageNotified)
		{
			parent->getViewSize (vSize);
			rect.offset (vSize.left, vSize.top);
		}
		parent = parent->getParentView ();
	}
	if (pContext)
		rect.offset (pContext->offset.h, pContext->offset.v);

	if (platformControl == 0) {
		int wstyle = 0;

		wstyle |= ES_LEFT;
		wstyle |= WS_CHILD | WS_VISIBLE | WS_VSCROLL | ES_AUTOVSCROLL | ES_MULTILINE;
		platformControl = (void*)CreateWindow (
			"EDIT", "", wstyle,
			rect.left, rect.top, rect.width ()/* + 1*/, rect.height ()/* + 1*/,
			(HWND)getFrame ()->getSystemWindow (), NULL, GetInstance (), 0);

		LOGFONT logfont = {0};

		long fontH = gStandardFontSize [fontID];
		if (fontH > rect.height () - 2)
			fontH = rect.height () - 2;

		logfont.lfWeight = FW_NORMAL;
		logfont.lfHeight = -fontH;
		logfont.lfPitchAndFamily = VARIABLE_PITCH | FF_SWISS;
		strcpy (logfont.lfFaceName, gStandardFontName[fontID]);

		logfont.lfClipPrecision	 = CLIP_STROKE_PRECIS;
		logfont.lfOutPrecision	 = OUT_STRING_PRECIS;
		logfont.lfQuality 	     = DEFAULT_QUALITY;
		logfont.lfCharSet        = SHIFTJIS_CHARSET;
	  
		platformFont = (HANDLE)CreateFontIndirect (&logfont);
		platformFontColor = 0;

		SetWindowLongPtr ((HWND)platformControl, GWLP_USERDATA, (LONG_PTR)this);
		SendMessage ((HWND)platformControl, WM_SETFONT, (WPARAM)platformFont, true);
		SendMessage ((HWND)platformControl, EM_SETMARGINS, EC_LEFTMARGIN|EC_RIGHTMARGIN, MAKELONG (0, 0));
		SendMessage ((HWND)platformControl, EM_LIMITTEXT, CTEXT_AREA_SIZE - 1, 0);
	}

	SetFocus ((HWND)platformControl);
}

//------------------------------------------------------------------------
void CTextArea::drawText (CDrawContext *pContext, char *string)
{
}

void CTextArea::setText (char *txt)
{
	strcpy (text, txt);
	setDirty ();
}

char* CTextArea::getText (void)
{
	GetWindowText((HWND)platformControl, text, CTEXT_AREA_SIZE);
	return text;
}
